/*----- PROTECTED REGION ID(A2720.cpp) ENABLED START -----*/
//=============================================================================
//
// file :        A2720.cpp
//
// description : C++ source for the A2720 class and its commands.
//               The class is derived from Device. It represents the
//               CORBA servant object which will be accessed from the
//               network. All commands which can be executed on the
//               A2720 are implemented in this file.
//
// project :     
//
// This file is part of Tango device class.
// 
// Tango is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// Tango is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Tango.  If not, see <http://www.gnu.org/licenses/>.
// 
// $Author: alessio $
//
// $Revision: 1.8 $
// $Date: 2019-05-06 08:21:47 $
//
// $HeadURL:  $
//
//=============================================================================
//                This file is generated by POGO
//        (Program Obviously used to Generate tango Object)
//=============================================================================


#include <A2720.h>
#include <A2720Class.h>

#define MAX_DEVSTRING_LENGTH 256

/*----- PROTECTED REGION END -----*/	//	A2720.cpp

/**
 *  A2720 class description:
 *    
 */

//================================================================
//  The following table gives the correspondence
//  between command and method names.
//
//  Command name  |  Method name
//================================================================
//  Abort         |  abort
//  Off           |  off
//  On            |  on
//  Reset         |  reset
//  StartCycling  |  start_cycling
//  Status        |  Inherited (no method)
//  State         |  Inherited (no method)
//================================================================

//================================================================
//  Attributes managed are:
//================================================================
//  ADCTempCoefficient0       |  Tango::DevFloat	Scalar
//  ADCTempCoefficient1       |  Tango::DevFloat	Scalar
//  ADCTemperature            |  Tango::DevFloat	Scalar
//  AuxVoltageCoefficient0    |  Tango::DevFloat	Scalar
//  AuxVoltageCoefficient1    |  Tango::DevFloat	Scalar
//  AuxiliaryVoltage          |  Tango::DevDouble	Scalar
//  CalibrationDate           |  Tango::DevString	Scalar
//  Current                   |  Tango::DevDouble	Scalar
//  CurrentCoefficient0       |  Tango::DevFloat	Scalar
//  CurrentCoefficient1       |  Tango::DevFloat	Scalar
//  CurrentCoefficient2       |  Tango::DevFloat	Scalar
//  CurrentCoefficient3       |  Tango::DevFloat	Scalar
//  CurrentCoefficient4       |  Tango::DevFloat	Scalar
//  CurrentCoefficient5       |  Tango::DevFloat	Scalar
//  CurrentMax                |  Tango::DevFloat	Scalar
//  CurrentMin                |  Tango::DevFloat	Scalar
//  CurrentSet                |  Tango::DevDouble	Scalar
//  DCCurrentCoefficient0     |  Tango::DevFloat	Scalar
//  DCCurrentCoefficient1     |  Tango::DevFloat	Scalar
//  DCLinkVoltage             |  Tango::DevDouble	Scalar
//  DCurr                     |  Tango::DevDouble	Scalar
//  HeatsinkTempCoefficient0  |  Tango::DevFloat	Scalar
//  HeatsinkTempCoefficient1  |  Tango::DevFloat	Scalar
//  HeatsinkTemperature       |  Tango::DevFloat	Scalar
//  KDCoefficient             |  Tango::DevFloat	Scalar
//  KFFCoefficient            |  Tango::DevFloat	Scalar
//  KICoefficient             |  Tango::DevFloat	Scalar
//  KPCoefficient             |  Tango::DevFloat	Scalar
//  Model                     |  Tango::DevString	Scalar
//  PulseDuration             |  Tango::DevFloat	Scalar
//  SerialNumber              |  Tango::DevString	Scalar
//  SlewRate                  |  Tango::DevFloat	Scalar
//  Voltage                   |  Tango::DevDouble	Scalar
//  VoltageCoefficient0       |  Tango::DevFloat	Scalar
//  VoltageCoefficient1       |  Tango::DevFloat	Scalar
//  PsStat                    |  Tango::DevBoolean	Spectrum  ( max = 10)
//================================================================

namespace A2720_ns
{
/*----- PROTECTED REGION ID(A2720::namespace_starting) ENABLED START -----*/

//	static initializations
template<typename F, typename T> void convert(const F &data, T &result)
{
	stringstream ss;
	ss << data;
	ss >> result;

	if (ss.fail()) {
		stringstream err_mesg;
		err_mesg << "Conversion fail of " << data;
		Tango::Except::throw_exception( "",
				err_mesg.str(),
				"convert()");
	}
}

template<typename F, typename T> void convert(const F &data, double &result)
{
	stringstream ss;
	ss << data;

	if (ss.str() == "Inf")
		result = INFINITY;
	else
		ss >> result;

	if (ss.fail()) {
		stringstream err_mesg;
		err_mesg << "Conversion fail of " << data;
		Tango::Except::throw_exception( "",
				err_mesg.str(),
				"convert()");
	}
}

template<typename F> void convert(const F &data, string &result)
{
	stringstream ss;
	ss << fixed;
	ss << data;
	result = ss.str();
	if (ss.fail()) {
		stringstream err_mesg;
		err_mesg << "Conversion fail of " << data;
		Tango::Except::throw_exception( "",
				err_mesg.str(),
				"convert()");
	}
}

/*----- PROTECTED REGION END -----*/	//	A2720::namespace_starting

//--------------------------------------------------------
/**
 *	Method     : A2720::A2720()
 *	Description: Constructors for a Tango device
 *                implementing the classA2720
 */
//--------------------------------------------------------
A2720::A2720(Tango::DeviceClass *cl, std::string &s)
 : Socket2Client(cl, s.c_str())
{
	/*----- PROTECTED REGION ID(A2720::constructor_1) ENABLED START -----*/
	init_device();
	/*----- PROTECTED REGION END -----*/	//	A2720::constructor_1
}
//--------------------------------------------------------
A2720::A2720(Tango::DeviceClass *cl, const char *s)
 : Socket2Client(cl, s)
{
	/*----- PROTECTED REGION ID(A2720::constructor_2) ENABLED START -----*/
	init_device();
	/*----- PROTECTED REGION END -----*/	//	A2720::constructor_2
}
//--------------------------------------------------------
A2720::A2720(Tango::DeviceClass *cl, const char *s, const char *d)
 : Socket2Client(cl, s, d)
{
	/*----- PROTECTED REGION ID(A2720::constructor_3) ENABLED START -----*/
	init_device();
	/*----- PROTECTED REGION END -----*/	//	A2720::constructor_3
}
//--------------------------------------------------------
A2720::~A2720()
{
	delete_device();
}

//--------------------------------------------------------
/**
 *	Method     : A2720::delete_device()
 *	Description: will be called at device destruction or at init command
 */
//--------------------------------------------------------
void A2720::delete_device()
{
	DEBUG_STREAM << "A2720::delete_device() " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::delete_device) ENABLED START -----*/
	
	//	Delete device allocated objects
	if (attr_Model_read[0])
		delete attr_Model_read[0];
	if (attr_CalibrationDate_read[0])
		delete attr_CalibrationDate_read[0];
	if (attr_SerialNumber_read[0])
		delete attr_SerialNumber_read[0];

	/*----- PROTECTED REGION END -----*/	//	A2720::delete_device
	delete[] attr_ADCTempCoefficient0_read;
	delete[] attr_ADCTempCoefficient1_read;
	delete[] attr_ADCTemperature_read;
	delete[] attr_AuxVoltageCoefficient0_read;
	delete[] attr_AuxVoltageCoefficient1_read;
	delete[] attr_AuxiliaryVoltage_read;
	delete[] attr_CalibrationDate_read;
	delete[] attr_Current_read;
	delete[] attr_CurrentCoefficient0_read;
	delete[] attr_CurrentCoefficient1_read;
	delete[] attr_CurrentCoefficient2_read;
	delete[] attr_CurrentCoefficient3_read;
	delete[] attr_CurrentCoefficient4_read;
	delete[] attr_CurrentCoefficient5_read;
	delete[] attr_CurrentMax_read;
	delete[] attr_CurrentMin_read;
	delete[] attr_CurrentSet_read;
	delete[] attr_DCCurrentCoefficient0_read;
	delete[] attr_DCCurrentCoefficient1_read;
	delete[] attr_DCLinkVoltage_read;
	delete[] attr_HeatsinkTempCoefficient0_read;
	delete[] attr_HeatsinkTempCoefficient1_read;
	delete[] attr_HeatsinkTemperature_read;
	delete[] attr_KDCoefficient_read;
	delete[] attr_KFFCoefficient_read;
	delete[] attr_KICoefficient_read;
	delete[] attr_KPCoefficient_read;
	delete[] attr_Model_read;
	delete[] attr_PulseDuration_read;
	delete[] attr_SerialNumber_read;
	delete[] attr_SlewRate_read;
	delete[] attr_Voltage_read;
	delete[] attr_VoltageCoefficient0_read;
	delete[] attr_VoltageCoefficient1_read;
	delete[] attr_PsStat_read;

	if (Tango::Util::instance()->is_svr_shutting_down()==false  &&
		Tango::Util::instance()->is_device_restarting(device_name)==false &&
		Tango::Util::instance()->is_svr_starting()==false)
	{
		//	If not shutting down call delete device for inherited object
		Socket2Client_ns::Socket2Client::delete_device();
	}
}

//--------------------------------------------------------
/**
 *	Method     : A2720::init_device()
 *	Description: will be called at device initialization.
 */
//--------------------------------------------------------
void A2720::init_device()
{
	DEBUG_STREAM << "A2720::init_device() create device " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::init_device_before) ENABLED START -----*/
	
	//	Initialization before get_device_property() call
	mandatoryNotDefined = false;
	init_error.clear();
	
	/*----- PROTECTED REGION END -----*/	//	A2720::init_device_before

	if (Tango::Util::instance()->is_svr_starting() == false  &&
		Tango::Util::instance()->is_device_restarting(device_name)==false)
	{
		//	If not starting up call init device for inherited object
		Socket2Client_ns::Socket2Client::init_device();
	}

	//	Get the device properties from database
	get_device_property();

	attr_ADCTempCoefficient0_read = new Tango::DevFloat[1];
	attr_ADCTempCoefficient1_read = new Tango::DevFloat[1];
	attr_ADCTemperature_read = new Tango::DevFloat[1];
	attr_AuxVoltageCoefficient0_read = new Tango::DevFloat[1];
	attr_AuxVoltageCoefficient1_read = new Tango::DevFloat[1];
	attr_AuxiliaryVoltage_read = new Tango::DevDouble[1];
	attr_CalibrationDate_read = new Tango::DevString[1];
	attr_Current_read = new Tango::DevDouble[1];
	attr_CurrentCoefficient0_read = new Tango::DevFloat[1];
	attr_CurrentCoefficient1_read = new Tango::DevFloat[1];
	attr_CurrentCoefficient2_read = new Tango::DevFloat[1];
	attr_CurrentCoefficient3_read = new Tango::DevFloat[1];
	attr_CurrentCoefficient4_read = new Tango::DevFloat[1];
	attr_CurrentCoefficient5_read = new Tango::DevFloat[1];
	attr_CurrentMax_read = new Tango::DevFloat[1];
	attr_CurrentMin_read = new Tango::DevFloat[1];
	attr_CurrentSet_read = new Tango::DevDouble[1];
	attr_DCCurrentCoefficient0_read = new Tango::DevFloat[1];
	attr_DCCurrentCoefficient1_read = new Tango::DevFloat[1];
	attr_DCLinkVoltage_read = new Tango::DevDouble[1];
	attr_HeatsinkTempCoefficient0_read = new Tango::DevFloat[1];
	attr_HeatsinkTempCoefficient1_read = new Tango::DevFloat[1];
	attr_HeatsinkTemperature_read = new Tango::DevFloat[1];
	attr_KDCoefficient_read = new Tango::DevFloat[1];
	attr_KFFCoefficient_read = new Tango::DevFloat[1];
	attr_KICoefficient_read = new Tango::DevFloat[1];
	attr_KPCoefficient_read = new Tango::DevFloat[1];
	attr_Model_read = new Tango::DevString[1];
	attr_PulseDuration_read = new Tango::DevFloat[1];
	attr_SerialNumber_read = new Tango::DevString[1];
	attr_SlewRate_read = new Tango::DevFloat[1];
	attr_Voltage_read = new Tango::DevDouble[1];
	attr_VoltageCoefficient0_read = new Tango::DevFloat[1];
	attr_VoltageCoefficient1_read = new Tango::DevFloat[1];
	attr_PsStat_read = new Tango::DevBoolean[10];
	//	No longer if mandatory property not set.
	if (mandatoryNotDefined)
		return;

	/*----- PROTECTED REGION ID(A2720::init_device) ENABLED START -----*/
	//	Initialize device
	try {
		attr_Model_read[0] = new char[MAX_DEVSTRING_LENGTH];
		attr_CalibrationDate_read[0] = new char[MAX_DEVSTRING_LENGTH];
		attr_SerialNumber_read[0] = new char[MAX_DEVSTRING_LENGTH];

		// Write PsStat's values property
		Tango::DbData dbdata;
		Tango::DbDatum dbdatum1("PsStat");
		dbdatum1 << (short)1;
		dbdata.push_back(dbdatum1);
		Tango::DbDatum dbdatum2("values");
	  	vector<string> str_values;
	  	str_values.push_back("ON");
	  	str_values.push_back("Over current fault");
	  	str_values.push_back("Crow bar fault");
	  	str_values.push_back("DCCT fault");
	  	str_values.push_back("Over temperature fault");
	  	str_values.push_back("Cross conduction fault");
	  	str_values.push_back("VAUX - under voltage");
	  	str_values.push_back("Magnet fault");
 		str_values.push_back("DC link fault actual");
	  	str_values.push_back("DC link warning history");
	  	str_values.push_back("Earth fault");
		dbdatum2 << str_values;
		dbdata.push_back(dbdatum2);
		db_dev->put_attribute_property(dbdata);

		// Waiting the device
		int retries = 0;
		while(get_state2() == 9 && retries++ < 5) {
			sleep(1);
		}	
	} catch (Tango::DevFailed &e) {
		init_error = "Initialization failed: " + string(e.errors[0].desc);
	} catch (...) {
		init_error = "Initialization failed: Unknown error";
	}

	if (! init_error.empty()) {
		ERROR_STREAM << init_error << endl;
		set_state(Tango::UNKNOWN);
		set_status(init_error);
		assert(false);
	}

	/*----- PROTECTED REGION END -----*/	//	A2720::init_device
}

//--------------------------------------------------------
/**
 *	Method     : A2720::get_device_property()
 *	Description: Read database to initialize property data members.
 */
//--------------------------------------------------------
void A2720::get_device_property()
{
	/*----- PROTECTED REGION ID(A2720::get_device_property_before) ENABLED START -----*/
	
	//	Initialize property data members
	
	/*----- PROTECTED REGION END -----*/	//	A2720::get_device_property_before

	mandatoryNotDefined = false;

	//	Read device properties from database.
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("CycEndCurr"));
	dev_prop.push_back(Tango::DbDatum("CycMaxCurr"));
	dev_prop.push_back(Tango::DbDatum("CycMinCurr"));
	dev_prop.push_back(Tango::DbDatum("CycNumTimes"));
	dev_prop.push_back(Tango::DbDatum("CycRampSlopeCurr"));
	dev_prop.push_back(Tango::DbDatum("CycWaitTime"));
	dev_prop.push_back(Tango::DbDatum("DeviceName"));

	//	is there at least one property to be read ?
	if (dev_prop.size()>0)
	{
		//	Call database and extract values
		if (Tango::Util::instance()->_UseDb==true)
			get_db_device()->get_property(dev_prop);

		//	get instance on A2720Class to get class property
		Tango::DbDatum	def_prop, cl_prop;
		A2720Class	*ds_class =
			(static_cast<A2720Class *>(get_device_class()));
		int	i = -1;

		//	Try to initialize CycEndCurr from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycEndCurr;
		else {
			//	Try to initialize CycEndCurr from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycEndCurr;
		}
		//	And try to extract CycEndCurr value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycEndCurr;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize CycMaxCurr from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycMaxCurr;
		else {
			//	Try to initialize CycMaxCurr from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycMaxCurr;
		}
		//	And try to extract CycMaxCurr value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycMaxCurr;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize CycMinCurr from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycMinCurr;
		else {
			//	Try to initialize CycMinCurr from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycMinCurr;
		}
		//	And try to extract CycMinCurr value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycMinCurr;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize CycNumTimes from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycNumTimes;
		else {
			//	Try to initialize CycNumTimes from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycNumTimes;
		}
		//	And try to extract CycNumTimes value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycNumTimes;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize CycRampSlopeCurr from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycRampSlopeCurr;
		else {
			//	Try to initialize CycRampSlopeCurr from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycRampSlopeCurr;
		}
		//	And try to extract CycRampSlopeCurr value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycRampSlopeCurr;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize CycWaitTime from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  cycWaitTime;
		else {
			//	Try to initialize CycWaitTime from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  cycWaitTime;
		}
		//	And try to extract CycWaitTime value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  cycWaitTime;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

		//	Try to initialize DeviceName from class property
		cl_prop = ds_class->get_class_property(dev_prop[++i].name);
		if (cl_prop.is_empty()==false)	cl_prop  >>  deviceName;
		else {
			//	Try to initialize DeviceName from default device value
			def_prop = ds_class->get_default_device_property(dev_prop[i].name);
			if (def_prop.is_empty()==false)	def_prop  >>  deviceName;
		}
		//	And try to extract DeviceName value from database
		if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  deviceName;
		//	Property StartDsPath is mandatory, check if has been defined in database.
		check_mandatory_property(cl_prop, dev_prop[i]);

	}

	/*----- PROTECTED REGION ID(A2720::get_device_property_after) ENABLED START -----*/
	
	//	Check device property data members init
	
	/*----- PROTECTED REGION END -----*/	//	A2720::get_device_property_after
}
//--------------------------------------------------------
/**
 *	Method     : A2720::check_mandatory_property()
 *	Description: For mandatory properties check if defined in database.
 */
//--------------------------------------------------------
void A2720::check_mandatory_property(Tango::DbDatum &class_prop, Tango::DbDatum &dev_prop)
{
	//	Check if all properties are empty
	if (class_prop.is_empty() && dev_prop.is_empty())
	{
		TangoSys_OMemStream	tms;
		tms << std::endl <<"Property \'" << dev_prop.name;
		if (Tango::Util::instance()->_UseDb==true)
			tms << "\' is mandatory but not defined in database";
		else
			tms << "\' is mandatory but cannot be defined without database";
		append_status(tms.str());
		mandatoryNotDefined = true;
		/*----- PROTECTED REGION ID(A2720::check_mandatory_property) ENABLED START -----*/
		cerr << tms.str() << " for " << device_name << endl;
		
		/*----- PROTECTED REGION END -----*/	//	A2720::check_mandatory_property
	}
}


//--------------------------------------------------------
/**
 *	Method     : A2720::always_executed_hook()
 *	Description: method always executed before any command is executed
 */
//--------------------------------------------------------
void A2720::always_executed_hook()
{
	DEBUG_STREAM << "A2720::always_executed_hook()  " << device_name << std::endl;
	if (mandatoryNotDefined)
	{
		Tango::Except::throw_exception(
					(const char *)"PROPERTY_NOT_SET",
					get_status().c_str(),
					(const char *)"A2720::always_executed_hook()");
	}
	/*----- PROTECTED REGION ID(A2720::always_executed_hook) ENABLED START -----*/
	
	//	code always executed before all requests
	if (! init_error.empty()) {
		set_state(Tango::UNKNOWN);
		set_status(init_error);
		return;
	}

	try {
		check_state();
	} catch (Tango::DevFailed &e) {
		set_state(Tango::FAULT);
		set_status(string(e.errors[0].desc));
	} catch (...) {
		set_state(Tango::FAULT);
		set_status("Unknown error");
	}

	/*----- PROTECTED REGION END -----*/	//	A2720::always_executed_hook
}

//--------------------------------------------------------
/**
 *	Method     : A2720::read_attr_hardware()
 *	Description: Hardware acquisition for attributes
 */
//--------------------------------------------------------
void A2720::read_attr_hardware(TANGO_UNUSED(std::vector<long> &attr_list))
{
	DEBUG_STREAM << "A2720::read_attr_hardware(std::vector<long> &attr_list) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_attr_hardware) ENABLED START -----*/
	
	//	Add your own code
	check_init();

	/*----- PROTECTED REGION END -----*/	//	A2720::read_attr_hardware
}
//--------------------------------------------------------
/**
 *	Method     : A2720::write_attr_hardware()
 *	Description: Hardware writing for attributes
 */
//--------------------------------------------------------
void A2720::write_attr_hardware(TANGO_UNUSED(std::vector<long> &attr_list))
{
	DEBUG_STREAM << "A2720::write_attr_hardware(std::vector<long> &attr_list) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::write_attr_hardware) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	A2720::write_attr_hardware
}

//--------------------------------------------------------
/**
 *	Read attribute ADCTempCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_ADCTempCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_ADCTempCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_ADCTempCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRTA0"), attr_ADCTempCoefficient0_read[0]);
	attr.set_value(attr_ADCTempCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_ADCTempCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute ADCTempCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_ADCTempCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_ADCTempCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_ADCTempCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWTA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_ADCTempCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute ADCTempCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_ADCTempCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_ADCTempCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_ADCTempCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRTA1"), attr_ADCTempCoefficient1_read[0]);
	attr.set_value(attr_ADCTempCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_ADCTempCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute ADCTempCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_ADCTempCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_ADCTempCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_ADCTempCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWTA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_ADCTempCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute ADCTemperature related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_ADCTemperature(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_ADCTemperature(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_ADCTemperature) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRT"), attr_ADCTemperature_read[0]);
	attr.set_value(attr_ADCTemperature_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_ADCTemperature
}
//--------------------------------------------------------
/**
 *	Read attribute AuxVoltageCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_AuxVoltageCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_AuxVoltageCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_AuxVoltageCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVXA0"), attr_AuxVoltageCoefficient0_read[0]);
	attr.set_value(attr_AuxVoltageCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_AuxVoltageCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute AuxVoltageCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_AuxVoltageCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_AuxVoltageCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_AuxVoltageCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVXA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_AuxVoltageCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute AuxVoltageCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_AuxVoltageCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_AuxVoltageCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_AuxVoltageCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVXA1"), attr_AuxVoltageCoefficient1_read[0]);
	attr.set_value(attr_AuxVoltageCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_AuxVoltageCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute AuxVoltageCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_AuxVoltageCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_AuxVoltageCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_AuxVoltageCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVXA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_AuxVoltageCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute AuxiliaryVoltage related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_AuxiliaryVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_AuxiliaryVoltage(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_AuxiliaryVoltage) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVX"), attr_AuxiliaryVoltage_read[0]);
	attr.set_value(attr_AuxiliaryVoltage_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_AuxiliaryVoltage
}
//--------------------------------------------------------
/**
 *	Read attribute CalibrationDate related method
 *
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CalibrationDate(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CalibrationDate(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CalibrationDate) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRCD"), attr_CalibrationDate_read[0]);
	attr.set_value(attr_CalibrationDate_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CalibrationDate
}
//--------------------------------------------------------
/**
 *	Read attribute Current related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_Current(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_Current(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_Current) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRI"), attr_Current_read[0]);
	attr.set_value(attr_Current_read);

	/*----- PROTECTED REGION END -----*/	//	A2720::read_Current
}
//--------------------------------------------------------
/**
 *	Write attribute Current related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_Current(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_Current(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_Current) ENABLED START -----*/
	check_init();
#if 0
	worker = new A2720Worker(this);
	worker->start();
#else
	string value;
	convert(w_val, value);
	send_command("MWI:" + value);
#endif
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_Current
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA0"), attr_CurrentCoefficient0_read[0]);
	attr.set_value(attr_CurrentCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA1"), attr_CurrentCoefficient1_read[0]);
	attr.set_value(attr_CurrentCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient2 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient2(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient2(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient2) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA2"), attr_CurrentCoefficient2_read[0]);
	attr.set_value(attr_CurrentCoefficient2_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient2
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient2 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient2(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient2(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient2) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA2:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient2
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient3 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient3(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient3(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient3) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA3"), attr_CurrentCoefficient3_read[0]);
	attr.set_value(attr_CurrentCoefficient3_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient3
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient3 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient3(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient3(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient3) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA3:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient3
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient4 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient4(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient4(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient4) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA4"), attr_CurrentCoefficient4_read[0]);
	attr.set_value(attr_CurrentCoefficient4_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient4
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient4 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient4(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient4(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient4) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA4:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient4
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentCoefficient5 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentCoefficient5(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentCoefficient5(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentCoefficient5) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIA5"), attr_CurrentCoefficient5_read[0]);
	attr.set_value(attr_CurrentCoefficient5_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentCoefficient5
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentCoefficient5 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentCoefficient5(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentCoefficient5(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentCoefficient5) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIA5:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentCoefficient5
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentMax related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentMax(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentMax(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentMax) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIMAX"), attr_CurrentMax_read[0]);
	attr.set_value(attr_CurrentMax_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentMax
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentMax related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentMax(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentMax(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentMax) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIMAX:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentMax
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentMin related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentMin(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentMin(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentMin) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIMIN"), attr_CurrentMin_read[0]);
	attr.set_value(attr_CurrentMin_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentMin
}
//--------------------------------------------------------
/**
 *	Write attribute CurrentMin related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_CurrentMin(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_CurrentMin(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_CurrentMin) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWIMIN:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_CurrentMin
}
//--------------------------------------------------------
/**
 *	Read attribute CurrentSet related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_CurrentSet(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_CurrentSet(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_CurrentSet) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRIS"), attr_CurrentSet_read[0]);
	attr.set_value(attr_CurrentSet_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_CurrentSet
}
//--------------------------------------------------------
/**
 *	Read attribute DCCurrentCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_DCCurrentCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_DCCurrentCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_DCCurrentCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVDCA0"), attr_DCCurrentCoefficient0_read[0]);
	attr.set_value(attr_DCCurrentCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_DCCurrentCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute DCCurrentCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_DCCurrentCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_DCCurrentCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_DCCurrentCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVDCA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_DCCurrentCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute DCCurrentCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_DCCurrentCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_DCCurrentCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_DCCurrentCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVDCA1"), attr_DCCurrentCoefficient1_read[0]);
	attr.set_value(attr_DCCurrentCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_DCCurrentCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute DCCurrentCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_DCCurrentCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_DCCurrentCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_DCCurrentCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVDCA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_DCCurrentCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute DCLinkVoltage related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_DCLinkVoltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_DCLinkVoltage(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_DCLinkVoltage) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVDC"), attr_DCLinkVoltage_read[0]);
	attr.set_value(attr_DCLinkVoltage_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_DCLinkVoltage
}
//--------------------------------------------------------
/**
 *	Write attribute DCurr related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_DCurr(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_DCurr(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevDouble	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_DCurr) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWI:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_DCurr
}
//--------------------------------------------------------
/**
 *	Read attribute HeatsinkTempCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_HeatsinkTempCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_HeatsinkTempCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_HeatsinkTempCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRTHA0"), attr_HeatsinkTempCoefficient0_read[0]);
	attr.set_value(attr_HeatsinkTempCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_HeatsinkTempCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute HeatsinkTempCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_HeatsinkTempCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_HeatsinkTempCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_HeatsinkTempCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWTHA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_HeatsinkTempCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute HeatsinkTempCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_HeatsinkTempCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_HeatsinkTempCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_HeatsinkTempCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRTHA1"), attr_HeatsinkTempCoefficient1_read[0]);
	attr.set_value(attr_HeatsinkTempCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_HeatsinkTempCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute HeatsinkTempCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_HeatsinkTempCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_HeatsinkTempCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_HeatsinkTempCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWTHA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_HeatsinkTempCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute HeatsinkTemperature related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_HeatsinkTemperature(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_HeatsinkTemperature(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_HeatsinkTemperature) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRTH"), attr_HeatsinkTemperature_read[0]);
	attr.set_value(attr_HeatsinkTemperature_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_HeatsinkTemperature
}
//--------------------------------------------------------
/**
 *	Read attribute KDCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_KDCoefficient(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_KDCoefficient(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_KDCoefficient) ENABLED START -----*/
	/* clang-format on */
	//	Set the attribute value
	convert(send_command("MRKD"), attr_KDCoefficient_read[0]);
	attr.set_value(attr_KDCoefficient_read);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	A2720::read_KDCoefficient
}
//--------------------------------------------------------
/**
 *	Write attribute KDCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_KDCoefficient(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_KDCoefficient(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_KDCoefficient) ENABLED START -----*/
	/* clang-format on */
	//	Add your own code
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWKD:" + value);
	/* clang-format off */
	/*----- PROTECTED REGION END -----*/	//	A2720::write_KDCoefficient
}
//--------------------------------------------------------
/**
 *	Read attribute KFFCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_KFFCoefficient(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_KFFCoefficient(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_KFFCoefficient) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRKFF"), attr_KFFCoefficient_read[0]);
	attr.set_value(attr_KFFCoefficient_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_KFFCoefficient
}
//--------------------------------------------------------
/**
 *	Write attribute KFFCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_KFFCoefficient(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_KFFCoefficient(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_KFFCoefficient) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWKFF:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_KFFCoefficient
}
//--------------------------------------------------------
/**
 *	Read attribute KICoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_KICoefficient(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_KICoefficient(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_KICoefficient) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRKI"), attr_KICoefficient_read[0]);
	attr.set_value(attr_KICoefficient_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_KICoefficient
}
//--------------------------------------------------------
/**
 *	Write attribute KICoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_KICoefficient(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_KICoefficient(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_KICoefficient) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWKI:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_KICoefficient
}
//--------------------------------------------------------
/**
 *	Read attribute KPCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_KPCoefficient(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_KPCoefficient(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_KPCoefficient) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRKP"), attr_KPCoefficient_read[0]);
	attr.set_value(attr_KPCoefficient_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_KPCoefficient
}
//--------------------------------------------------------
/**
 *	Write attribute KPCoefficient related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_KPCoefficient(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_KPCoefficient(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_KPCoefficient) ENABLED START -----*/
	check_init();

	string value;
	convert(w_val, value);
	send_command("MWKP:" + value);

	/*----- PROTECTED REGION END -----*/	//	A2720::write_KPCoefficient
}
//--------------------------------------------------------
/**
 *	Read attribute Model related method
 *
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_Model(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_Model(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_Model) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRID"), attr_Model_read[0]);
	attr.set_value(attr_Model_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_Model
}
//--------------------------------------------------------
/**
 *	Read attribute PulseDuration related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_PulseDuration(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_PulseDuration(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_PulseDuration) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRPD"), attr_PulseDuration_read[0]);
	attr.set_value(attr_PulseDuration_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_PulseDuration
}
//--------------------------------------------------------
/**
 *	Write attribute PulseDuration related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_PulseDuration(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_PulseDuration(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_PulseDuration) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWPD:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_PulseDuration
}
//--------------------------------------------------------
/**
 *	Read attribute SerialNumber related method
 *
 *
 *	Data type:	Tango::DevString
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_SerialNumber(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_SerialNumber(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_SerialNumber) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRSN"), attr_SerialNumber_read[0]);
	attr.set_value(attr_SerialNumber_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_SerialNumber
}
//--------------------------------------------------------
/**
 *	Read attribute SlewRate related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_SlewRate(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_SlewRate(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_SlewRate) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRSR"), attr_SlewRate_read[0]);
	attr.set_value(attr_SlewRate_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_SlewRate
}
//--------------------------------------------------------
/**
 *	Write attribute SlewRate related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_SlewRate(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_SlewRate(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_SlewRate) ENABLED START -----*/
	check_init();

	string value;
	convert(w_val, value);
	send_command("MWSR:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_SlewRate
}
//--------------------------------------------------------
/**
 *	Read attribute Voltage related method
 *
 *
 *	Data type:	Tango::DevDouble
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_Voltage(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_Voltage(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_Voltage) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRV"), attr_Voltage_read[0]);
	attr.set_value(attr_Voltage_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_Voltage
}
//--------------------------------------------------------
/**
 *	Read attribute VoltageCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_VoltageCoefficient0(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_VoltageCoefficient0(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_VoltageCoefficient0) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVA0"), attr_VoltageCoefficient0_read[0]);
	attr.set_value(attr_VoltageCoefficient0_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_VoltageCoefficient0
}
//--------------------------------------------------------
/**
 *	Write attribute VoltageCoefficient0 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_VoltageCoefficient0(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_VoltageCoefficient0(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_VoltageCoefficient0) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVA0:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_VoltageCoefficient0
}
//--------------------------------------------------------
/**
 *	Read attribute VoltageCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::read_VoltageCoefficient1(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_VoltageCoefficient1(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_VoltageCoefficient1) ENABLED START -----*/
	//	Set the attribute value
	convert(send_command("MRVA1"), attr_VoltageCoefficient1_read[0]);
	attr.set_value(attr_VoltageCoefficient1_read);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_VoltageCoefficient1
}
//--------------------------------------------------------
/**
 *	Write attribute VoltageCoefficient1 related method
 *
 *
 *	Data type:	Tango::DevFloat
 *	Attr type:	Scalar
 */
//--------------------------------------------------------
void A2720::write_VoltageCoefficient1(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "A2720::write_VoltageCoefficient1(Tango::WAttribute &attr) entering... " << std::endl;
	//	Retrieve write value
	Tango::DevFloat	w_val;
	attr.get_write_value(w_val);
	/*----- PROTECTED REGION ID(A2720::write_VoltageCoefficient1) ENABLED START -----*/
	check_init();
	
	string value;
	convert(w_val, value);
	send_command("MWVA1:" + value);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::write_VoltageCoefficient1
}
//--------------------------------------------------------
/**
 *	Read attribute PsStat related method
 *
 *
 *	Data type:	Tango::DevBoolean
 *	Attr type:	Spectrum max = 10
 */
//--------------------------------------------------------
void A2720::read_PsStat(Tango::Attribute &attr)
{
	DEBUG_STREAM << "A2720::read_PsStat(Tango::Attribute &attr) entering... " << std::endl;
	/*----- PROTECTED REGION ID(A2720::read_PsStat) ENABLED START -----*/
	//	Set the attribute value
	memset(attr_PsStat_read, 0, sizeof(Tango::DevBoolean) * 10);

	if (~iostatus & 0x1)
		attr_PsStat_read[0] = true;

	if (iostatus & 0x4)
		attr_PsStat_read[1] = true;

	if (iostatus & 0x8)
		attr_PsStat_read[2] = true;

	if (iostatus & 0x40)
		attr_PsStat_read[3] = true;

	if (iostatus & 0x80)
		attr_PsStat_read[4] = true;

	if (iostatus & 0x1000)
		attr_PsStat_read[5] = true;

	if (iostatus & 0x2000)
		attr_PsStat_read[6] = true;

	if (iostatus & 0x8000)
		attr_PsStat_read[7] = true;

	if (~iostatus & 0x1000000)
		attr_PsStat_read[8] = true;

	if (~iostatus & 0x2000000)
		attr_PsStat_read[9] = true;

	if (iostatus & 0x4000000)
		attr_PsStat_read[10] = true;

	attr.set_value(attr_PsStat_read, 10);
	
	/*----- PROTECTED REGION END -----*/	//	A2720::read_PsStat
}

//--------------------------------------------------------
/**
 *	Method     : A2720::add_dynamic_attributes()
 *	Description: Create the dynamic attributes if any
 *                for specified device.
 */
//--------------------------------------------------------
void A2720::add_dynamic_attributes()
{
	/*----- PROTECTED REGION ID(A2720::add_dynamic_attributes) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic attributes if any
	
	/*----- PROTECTED REGION END -----*/	//	A2720::add_dynamic_attributes
}

//--------------------------------------------------------
/**
 *	Command Abort related method
 *
 *
 */
//--------------------------------------------------------
void A2720::abort()
{
	DEBUG_STREAM << "A2720::Abort()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::abort) ENABLED START -----*/
	
	//	Add your own code
	check_init();
#if 0
	cont = false;
#endif
	
	/*----- PROTECTED REGION END -----*/	//	A2720::abort
}
//--------------------------------------------------------
/**
 *	Command Off related method
 *
 *
 */
//--------------------------------------------------------
void A2720::off()
{
	DEBUG_STREAM << "A2720::Off()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::off) ENABLED START -----*/
	
	//	Add your own code
	check_init();
	send_command("MOFF");

	/*----- PROTECTED REGION END -----*/	//	A2720::off
}
//--------------------------------------------------------
/**
 *	Command On related method
 *
 *
 */
//--------------------------------------------------------
void A2720::on()
{
	DEBUG_STREAM << "A2720::On()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::on) ENABLED START -----*/
	
	//	Add your own code
	check_init();
	send_command("MON");

	/*----- PROTECTED REGION END -----*/	//	A2720::on
}
//--------------------------------------------------------
/**
 *	Command Reset related method
 *
 *
 */
//--------------------------------------------------------
void A2720::reset()
{
	DEBUG_STREAM << "A2720::Reset()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::reset) ENABLED START -----*/
	
	//	Add your own code
	check_init();
	send_command("MRESET");

	/*----- PROTECTED REGION END -----*/	//	A2720::reset
}
//--------------------------------------------------------
/**
 *	Command StartCycling related method
 *
 *
 */
//--------------------------------------------------------
void A2720::start_cycling()
{
	DEBUG_STREAM << "A2720::StartCycling()  - " << device_name << std::endl;
	/*----- PROTECTED REGION ID(A2720::start_cycling) ENABLED START -----*/
	
	//	Add your own code
	check_init();
	Tango::Except::throw_exception( "",
			"Not implemented yet",
			"A2720::start_cycling()");

	/*----- PROTECTED REGION END -----*/	//	A2720::start_cycling
}
//--------------------------------------------------------
/**
 *	Method     : A2720::add_dynamic_commands()
 *	Description: Create the dynamic commands if any
 *                for specified device.
 */
//--------------------------------------------------------
void A2720::add_dynamic_commands()
{
	/*----- PROTECTED REGION ID(A2720::add_dynamic_commands) ENABLED START -----*/
	
	//	Add your own code to create and add dynamic commands if any
	
	/*----- PROTECTED REGION END -----*/	//	A2720::add_dynamic_commands
}

/*----- PROTECTED REGION ID(A2720::namespace_ending) ENABLED START -----*/

//	Additional Methods
void A2720::check_init()
{
	if (! init_error.empty())
		Tango::Except::throw_exception("",
				init_error.c_str(),
				"A2720::check_init()");
}

string A2720::send_command(const string &cmd)
{
	Socket2Client::write(cmd + "\r\n");

	string response, discard, reply;
	Socket2Client::read_until('\r', response);
	Socket2Client::read(1, discard);
	if (response[0] != '#')
		Tango::Except::throw_exception("",
				"First char in reply isn't a '#'!",
				"A2720::send_command()");
	reply = response.substr(1, response.size()-2);
	if (reply == "NAK")
		Tango::Except::throw_exception("",
				"Receive an non-acknowledgment",
				"A2720::send_command()");
	if (reply == "AK")
		return "";
	return reply;
}

void A2720::check_state()
{
	string reply = send_command("MRST");
	convert(reply, iostatus);
	if (~iostatus & 0x1)
		set_state(Tango::ON);
	else
		set_state(Tango::OFF);
	if (iostatus > 1)
		set_state(Tango::FAULT);
}

#if 0
bool A2720::is_on_target()
{
	float current, setpoint;
	
	convert(send_command("MRI"), current);
	convert(send_command("MRIS"), setpoint);

	if (current < setpoint + 0.002 && current > setpoint - 0.002)
		return true;
	return false;

}
#endif

#if 0
void A2720Worker::run(void*)
{
	DEBUG_STREAM << "A2720Worker::run() - enter" << endl;

	while(device->cont) {
		if (device->is_on_target())
			break;
		omni_thread::self()->sleep(0, 50 * 1000000);
	}

	DEBUG_STREAM << "A2720Worker::run() - exit" << endl;
}
#endif

/*----- PROTECTED REGION END -----*/	//	A2720::namespace_ending
} //	namespace
